#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_DOSDEVIOCTL
#include <os2.h>
#include "util.h"

#define OEMHLP_NAME "OEMHLP$"
#define BUSNUM(x)  ((UCHAR)(x >> 8))
#define DEVFUNC(x) ((UCHAR)(x & 0xff))

#pragma pack (1)

typedef struct _PCI_PARM {
   UCHAR PCISubFunc;
   union {
      struct {
         USHORT DeviceID;
         USHORT VendorID;
         UCHAR  Index;
      }Parm_Find_Dev;
      struct {
         ULONG  ClassCode;
         UCHAR  Index;
      }Parm_Find_ClassCode;
      struct {
         UCHAR  BusNum;
         UCHAR  DevFunc;
         UCHAR  ConfigReg;
         UCHAR  Size;
      }Parm_Read_Config;
      struct {
         UCHAR  BusNum;
         UCHAR  DevFunc;
         UCHAR  ConfigReg;
         UCHAR  Size;
         ULONG  Data;
      }Parm_Write_Config;
   };
} PCI_PARM;

typedef struct _PCI_DATA {
   UCHAR bReturn;
   union {
      struct {
         UCHAR HWMech;
         UCHAR MajorVer;
         UCHAR MinorVer;
         UCHAR LastBus;
      } Data_Bios_Info;
      struct {
         UCHAR  BusNum;
         UCHAR  DevFunc;
      }Data_Find_Dev;
      struct {
         ULONG  Data;
      }Data_Read_Config;
   };
} PCI_DATA;



ULONG _System PciQueryVersion  (USHORT * version) {
PCI_PARM parm = {0};
PCI_DATA data = {0};
ULONG act, parm_size,data_size,rc;
HFILE hdev;
     rc = DosOpen(OEMHLP_NAME,
                  &hdev,
                  &act,
                  0,
                  FILE_SYSTEM,
                  OPEN_ACTION_OPEN_IF_EXISTS,
                  OPEN_FLAGS_WRITE_THROUGH |
                  OPEN_SHARE_DENYNONE |
                  OPEN_ACCESS_READWRITE,
                  0 );
     if (rc != NO_ERROR )
        return rc;

     parm.PCISubFunc = 0x00;
     parm_size = sizeof(parm);
     data_size = sizeof(data);
     rc = DosDevIOCtl(hdev,
                      IOCTL_OEMHLP,
                      OEMHLP_PCI,
                      &parm,
                      parm_size,
                      &parm_size,
                      &data,
                      data_size,
                      &data_size);
     DosClose(hdev);
     if (rc != NO_ERROR )
        return rc;
     *version = (data.Data_Bios_Info.MajorVer << 8) | (data.Data_Bios_Info.MinorVer);
     return (ULONG)data.bReturn;

}

ULONG _System PciFindDevice    (USHORT VendorID, USHORT DeviceID, UCHAR Index, USHORT * addr) {
PCI_PARM parm = {0};
PCI_DATA data = {0};
ULONG act, parm_size,data_size,rc;
HFILE hdev;
     rc = DosOpen(OEMHLP_NAME,
                  &hdev,
                  &act,
                  0,
                  FILE_SYSTEM,
                  OPEN_ACTION_OPEN_IF_EXISTS,
                  OPEN_FLAGS_WRITE_THROUGH |
                  OPEN_SHARE_DENYNONE |
                  OPEN_ACCESS_READWRITE,
                  0 );
     if (rc != NO_ERROR )
        return rc;
     parm.PCISubFunc = 0x01;
     parm.Parm_Find_Dev.VendorID = VendorID;
     parm.Parm_Find_Dev.DeviceID = DeviceID;
     parm.Parm_Find_Dev.Index = Index;
     parm_size = sizeof(parm);
     data_size = sizeof(data);
     rc = DosDevIOCtl(hdev,
                      IOCTL_OEMHLP,
                      OEMHLP_PCI,
                      &parm,
                      parm_size,
                      &parm_size,
                      &data,
                      data_size,
                      &data_size);
     DosClose(hdev);
     if (rc != NO_ERROR )
        return rc;
     *addr = (data.Data_Find_Dev.BusNum<<8) | (data.Data_Find_Dev.DevFunc);
     return (ULONG)data.bReturn;
}

ULONG _System PciFindClass     (ULONG ClassCode, UCHAR Index, USHORT * addr) {
PCI_PARM parm = {0};
PCI_DATA data = {0};
ULONG act, parm_size,data_size,rc;
HFILE hdev;
     rc = DosOpen(OEMHLP_NAME,
                  &hdev,
                  &act,
                  0,
                  FILE_SYSTEM,
                  OPEN_ACTION_OPEN_IF_EXISTS,
                  OPEN_FLAGS_WRITE_THROUGH |
                  OPEN_SHARE_DENYNONE |
                  OPEN_ACCESS_READWRITE,
                  0 );
     if (rc != NO_ERROR )
        return rc;
     parm.PCISubFunc = 0x02;
     parm.Parm_Find_ClassCode.ClassCode = ClassCode;
     parm.Parm_Find_ClassCode.Index = Index;
     parm_size = sizeof(parm);
     data_size = sizeof(data);
     rc = DosDevIOCtl(hdev,
                      IOCTL_OEMHLP,
                      OEMHLP_PCI,
                      &parm,
                      parm_size,
                      &parm_size,
                      &data,
                      data_size,
                      &data_size);
     DosClose(hdev);
     if (rc != NO_ERROR )
        return rc;
     *addr = (data.Data_Find_Dev.BusNum<<8) | (data.Data_Find_Dev.DevFunc);
     return (ULONG)data.bReturn;
}

ULONG _System PciReadCfgDword  (USHORT addr, UCHAR reg, ULONG *Data) {
PCI_PARM parm = {0};
PCI_DATA data = {0};
ULONG act, parm_size,data_size,rc;
HFILE hdev;
     rc = DosOpen(OEMHLP_NAME,
                  &hdev,
                  &act,
                  0,
                  FILE_SYSTEM,
                  OPEN_ACTION_OPEN_IF_EXISTS,
                  OPEN_FLAGS_WRITE_THROUGH |
                  OPEN_SHARE_DENYNONE |
                  OPEN_ACCESS_READWRITE,
                  0 );
     if (rc != NO_ERROR )
        return rc;
     parm.PCISubFunc = 0x03;
     parm.Parm_Read_Config.BusNum = BUSNUM(addr);
     parm.Parm_Read_Config.DevFunc = DEVFUNC(addr);
     parm.Parm_Read_Config.ConfigReg = reg;
     parm.Parm_Read_Config.Size = 4;
     parm_size = sizeof(parm);
     data_size = sizeof(data);
     rc = DosDevIOCtl(hdev,
                      IOCTL_OEMHLP,
                      OEMHLP_PCI,
                      &parm,
                      parm_size,
                      &parm_size,
                      &data,
                      data_size,
                      &data_size);
     DosClose(hdev);
     if (rc != NO_ERROR )
        return rc;
     * Data = data.Data_Read_Config.Data;
     return (ULONG)data.bReturn;
}

ULONG _System PciReadCfgByte   (USHORT addr, UCHAR reg, UCHAR *Data) {
PCI_PARM parm = {0};
PCI_DATA data = {0};
ULONG act, parm_size,data_size,rc;
HFILE hdev;
     rc = DosOpen(OEMHLP_NAME,
                  &hdev,
                  &act,
                  0,
                  FILE_SYSTEM,
                  OPEN_ACTION_OPEN_IF_EXISTS,
                  OPEN_FLAGS_WRITE_THROUGH |
                  OPEN_SHARE_DENYNONE |
                  OPEN_ACCESS_READWRITE,
                  0 );
     if (rc != NO_ERROR )
        return rc;
     parm.PCISubFunc = 0x03;
     parm.Parm_Read_Config.BusNum = BUSNUM(addr);
     parm.Parm_Read_Config.DevFunc = DEVFUNC(addr);
     parm.Parm_Read_Config.ConfigReg = reg;
     parm.Parm_Read_Config.Size = 1;
     parm_size = sizeof(parm);
     data_size = sizeof(data);
     rc = DosDevIOCtl(hdev,
                      IOCTL_OEMHLP,
                      OEMHLP_PCI,
                      &parm,
                      parm_size,
                      &parm_size,
                      &data,
                      data_size,
                      &data_size);
     DosClose(hdev);
     if (rc != NO_ERROR )
        return rc;
     * Data = (UCHAR)data.Data_Read_Config.Data;
     return (ULONG)data.bReturn;
}

ULONG _System PciWriteCfgDword (USHORT addr, UCHAR reg, ULONG Data) {
PCI_PARM parm = {0};
PCI_DATA data = {0};
ULONG act, parm_size,data_size,rc;
HFILE hdev;
     rc = DosOpen(OEMHLP_NAME,
                  &hdev,
                  &act,
                  0,
                  FILE_SYSTEM,
                  OPEN_ACTION_OPEN_IF_EXISTS,
                  OPEN_FLAGS_WRITE_THROUGH |
                  OPEN_SHARE_DENYNONE |
                  OPEN_ACCESS_READWRITE,
                  0 );
     if (rc != NO_ERROR )
        return rc;
     parm.PCISubFunc = 0x04;
     parm.Parm_Write_Config.BusNum = BUSNUM(addr);
     parm.Parm_Write_Config.DevFunc = DEVFUNC(addr);

     parm.Parm_Write_Config.ConfigReg = reg;
     parm.Parm_Write_Config.Size = 4;
     parm.Parm_Write_Config.Data = Data;
     parm_size = sizeof(parm);
     data_size = sizeof(data);
     rc = DosDevIOCtl(hdev,
                      IOCTL_OEMHLP,
                      OEMHLP_PCI,
                      &parm,
                      parm_size,
                      &parm_size,
                      &data,
                      data_size,
                      &data_size);
     DosClose(hdev);
     if (rc != NO_ERROR )
        return rc;
     return (ULONG)data.bReturn;
}

ULONG _System PciWriteCfgByte  (USHORT addr, UCHAR reg, UCHAR Data) {
PCI_PARM parm = {0};
PCI_DATA data = {0};
ULONG act, parm_size,data_size,rc;
HFILE hdev;
     rc = DosOpen(OEMHLP_NAME,
                  &hdev,
                  &act,
                  0,
                  FILE_SYSTEM,
                  OPEN_ACTION_OPEN_IF_EXISTS,
                  OPEN_FLAGS_WRITE_THROUGH |
                  OPEN_SHARE_DENYNONE |
                  OPEN_ACCESS_READWRITE,
                  0 );
     if (rc != NO_ERROR )
        return rc;
     parm.PCISubFunc = 0x04;
     parm.Parm_Write_Config.BusNum = BUSNUM(addr);
     parm.Parm_Write_Config.DevFunc = DEVFUNC(addr);
     parm.Parm_Write_Config.ConfigReg = reg;
     parm.Parm_Write_Config.Size = 1;
     parm.Parm_Write_Config.Data = Data;
     parm_size = sizeof(parm);
     data_size = sizeof(data);
     rc = DosDevIOCtl(hdev,
                      IOCTL_OEMHLP,
                      OEMHLP_PCI,
                      &parm,
                      parm_size,
                      &parm_size,
                      &data,
                      data_size,
                      &data_size);
     DosClose(hdev);
     if (rc != NO_ERROR )
        return rc;
     return (ULONG)data.bReturn;
}

